#!/bin/sh
# @Description:
# 	Script helps Qt assistant to display man files.
# @Author:
#	Suetnov Kirill
# @Date:
#	16.03.2007

#alex: check for standard man2html first
if [ ! -e /usr/bin/man2html ]; then

if [ -z "$QTDIR" ]; then
    exit 2
fi

if [ ! -e $QTDIR/bin/qman2html ]; then
    exit 3
fi

fi 

input="$1"

if [ -z "$input" ]; then
    exit 4
fi

homepath="$HOME/.assistant/man.cached"
tmppath="$TMPDIR/.assistant/man.cached"

outname=`basename $1 | sed 's,.gz$,,g'`.html
#outname="res.html"
#1. check if we have cached version if required page
# Look in: $HOME/.assistant/man.cached
# $TMPDIR/.assistant/man.cached
if [ -d "$homepath" ]; then
    if [ -e $homepath/$outname ]; then
	echo $homepath/$outname
	exit 0
    fi	
fi    

#ok, look in $TMPDIR
if [ -d $tmppath ]; then
    if [ -e $tmppath/$outname ]; then
	echo $tmppath/$outname
	exit 0
    fi	
fi	

targetpath=""
found="no"

#Nothing. So, let's try to convert. Let's detect writable directory
if [ ! -d $homepath ]; then
    mkdir -p $homepath >/dev/null 2>&1
fi    
    
if echo "">$homepath/__wrtest__ 2>/dev/null; then
    targetpath=$homepath
    rm -f $homepath/__wrtest__
else
    if [ ! -d $tmppath ]; then
	mkdir -p $homepath >/dev/null 2>&1
    fi    
    if echo "">$tmppath/__wrtest__ 2>/dev/null; then    
	targetpath=$tmppath
	rm -f $tmppath/__wrtest__
    else
	#Smth. strange. May be, tmp? More simple?
	if echo "">/tmp/__wrtest__; then    
	    targetpath=/tmp
	    rm -f /tmp/__wrtest__
	else
	    exit 5    
	fi    
    fi
fi    

#Ok, do actual convert - first of all, gunzip it, then - run through qman2html
gunzip -c -q $input > $targetpath/$outname.man 2>/dev/null
#-L $targetpath
#$QTDIR/bin/qman2html -L "" -M $targetpath -p $targetpath/$outname.man \

#-M switch tells qman2html to prepend given path before each link.
#But the path depends on command type. So, we will replace it later.
#&LINK/5+aide.conf

tmpfile=`mktemp $targetpath/$outname.tmp.XXXXXX`
if [ -e $QTDIR/bin/qman2html ]; then
    man2html_prg=$QTDIR/bin/qman2html
else
    man2html_prg="man2html"
fi
        
$man2html_prg -L "" -M "&LINK" -p $targetpath/$outname.man 2>/dev/null > $tmpfile

if grep -e '<HTML><HEAD><TITLE>Invalid Manpage</TITLE></HEAD>' $tmpfile >/dev/null; then
cat > $targetpath/$outname <<EOF
<HTML><HEAD><TITLE>Ошибка преобразования man-страницы</TITLE></HEAD>
<BODY>
<H1>Man-страница имеет неверный формат или отсутствует</H1>
Запрошенный файл не может быть отображен, так как имеет неверный формат или отсутствует.
</BODY></HTML>
EOF
else
cat $tmpfile | sed -e '1d' -e "s,lynxcgi:,,g" | gawk  \
    'function removeHref() \
    {
	#print "Func_arg=",$0;
    };
    { \
    
    if ( $0 ~ "HREF") \
    { \
	pos=match($0,/".*"/);
	lng=RLENGTH;
	rpath=substr($0,pos,lng);
	if ( substr(rpath,1,7)=="\"\&LINK/") \
	{
	    print "<!--Input rpath=",rpath,"-->";
	    man_num=gensub(/"&LINK\/([[:digit:]]+)\+[[:alnum:][:punct:]]+"/,"\\1","g",rpath);
	    man_name=gensub(/"&LINK\/[[:digit:]]+\+([[:alnum:][:punct:]]+)"/,"\\1","g",rpath);
	    print "<!--man_num=|",man_num,"|-->";
	    print "<!--man_name=|",man_name,"|-->";
	    raw_locale=ENVIRON["LANG"];
	    locale=substr(raw_locale,1,2);
	    #Lets find this man page in common dirctories
	    searchpath[1]="/usr/man/" locale "/man" man_num;
	    searchpath[2]="/usr/share/man/" locale "/man" man_num;
	    searchpath[3]="/usr/man/man" man_num;
	    searchpath[4]="/usr/share/man/man" man_num;
	    fflag="false";
	    for (i=1; i<=4;++i) \
	    {
		syspath=searchpath[i];
		syscommand="if [ -e " syspath "/" man_name "." man_num ".gz ]; then exit 0; else exit 1; fi;";
		res=system(syscommand);
		#print "RES=",res;
		if (res=="0") \
		{
		    #Ok, we found optimal man page. Lets correct link.
		    good_link=gensub(/"&LINK\/([[:digit:]]+)\+([[:alnum:][:punct:]]+)"/,syspath "/" "\\2.\\1.gz","g",rpath);
		    gsub(/".*"/,good_link);
		    fflag="true";
		    #print "GOOD_LINK=",good_link;
		} 
		else
		{
		    #msg="<!--File not found: " syspath "/" man_name "." man_num ".gz -->"
		    #print msg;
		}
	    }
	    if (fflag=="false")	\
	    {
		gsub(/HREF[[:space:]]*=[[:space:]]*".*"/," ");
	    }
	    #print "SYSCMD=",syscommand;
	}
	else if ( substr(rpath,1,6)=="\"\&LINK") \
	{
	    gsub(/HREF[[:space:]]*=[[:space:]]*".*"/," ");
	}
    } \
    print $0; 
}'> $targetpath/$outname
fi

rm -f $targetpath/$outname.man
rm -f $tmpfile
echo $targetpath/$outname
exit 0




